#!/usr/bin/perl -X
use DBI;
use Socket;
use Term::ANSIColor qw(:constants);
use POSIX qw/strftime/;
use Config::IniFiles;
use Data::Dumper;
use Time::Local;
use Getopt::Long;
use Mail::Sender;
use Email::MessageID;
use MIME::QuotedPrint;
use LWP::UserAgent;

my $p_version = "1.0.0.PL";
my $p_data = "13.09.2014";
my $STARTA = time();

my %options = (
	"--quiet|q"		=> \my $argGuiet,
	"--help|h|?"		=> \my $argHelp,
	"--version|v"		=> \my $argVersion,
	"--config-file|C=s"	=> \my $configfile,
	"--ping|p"		=> \my $argPing,
	"--alert-mail|am"	=> \my $argAlertMail,
	"--alert-tel|at"	=> \my $argAlertTel,
	"--new|n"		=> \my $argNew,
);
Getopt::Long::config("no_ignore_case");
GetOptions(%options);

   if($argHelp){
	print " Alert for LMS, version $p_version [$p_data]\n";
	print " autor: seszu (seba\@kazuko.pl)\n";
	print "\n";
	print "-q,  --quiet		tłumi dane na wyjściu; error, done\n";
	print "-h,  --help		drukuje pomoc\n";
	print "-C,  --config-file	alternatywny plik konfiguracji		(default: /etc/lms/lms.ini)\n";
	print "\n";
	print "-p,  --ping		sprawdza czy hosty są osiagalne (fping -ar3 *HOST*)\n";
	print "-am, --alert-mail	jeśli trzeba wysyła MAILa z alertem\n";
	print "-at, --alert-tel	jeśli trzeba wysyła SMSa z alertem przez modem\n";
	print "-n,  --new		sprawdza czy jest nowa wersja (potrzebny dostęp do internetu)\n";
	print "\n";
	exit 0;
   }

   if($argVersion) {
	print " Alert for LMS, version $p_version [$p_data]\n";
	print " autor: seszu (seba\@kazuko.pl)\n";
	print " strona: http://kazuko.pl\n";
	print "\n";
	exit 0;
   }

   if($argNew) {
	print " Alert for LMS, version $p_version [$p_data]\n";
	print " autor: seszu (seba\@kazuko.pl)\n";
	print " strona: http://kazuko.pl\n";
	print "\n\n";

	my $ua = LWP::UserAgent->new; $ua->agent("Alert_for_LMS/$p_version/$p_data ");


	my $reqCurrent = HTTP::Request->new(POST => "http://script.kazuko.pl/Alert_for_LMS/version_current");
	$reqCurrent->content_type('application/x-www-form-urlencoded'); $reqCurrent->content('query=libwww-perl&mode=dist');
	$resCurrent = $ua->request($reqCurrent);

	my $reqPrevious = HTTP::Request->new(POST => "http://script.kazuko.pl/Alert_for_LMS/version_previous");
	$reqPrevious->content_type('application/x-www-form-urlencoded'); $reqPrevious->content('query=libwww-perl&mode=dist');
	$resPrevious = $ua->request($reqPrevious);


	   if($resCurrent->is_success and $resPrevious->is_success) {

		my $zCurrent = my @aCurrent = split(/\n/, $resCurrent->content);
		my @aPrevious = split(/\n/, $resPrevious->content);
		$p_data =~ s/\./\-/g;
		my $dataOld = $p_data;
		my $dataNew = $aCurrent[1];

		   if($p_data  =~ /^\d{2,2}\-\d{2,2}\-\d{4,4}$/) {
			my ($dd, $mm, $rrrr) = split(/\-/, $p_data);
			$dataOld = "$rrrr-$mm-$dd";
		   }
		   if($aCurrent[1]  =~ /^\d{2,2}\-\d{2,2}\-\d{4,4}$/) {
			my ($dd, $mm, $rrrr) = split(/\-/, $aCurrent[1]);
			$dataNew = "$rrrr-$mm-$dd";
		   }


		print " ".lang('arg', 'your-version').":\t\t$p_version [$p_data]\n";
		print " ".lang('arg', 'latest-version').":\t".$aCurrent[0]." [".$aCurrent[1]."]\n";
		print " ".lang('arg', 'previous-version').":\t".$aPrevious[0]." [".$aPrevious[1]."]\n";
		print " ---------------------------------------------------\n";

		   if(str2time($dataOld) >= str2time($dataNew)) {
			print BLUE, " ".lang('arg', 'current-versions').".\n", RESET;
		   } else {
			print GREEN, " ".lang('arg', 'new-version'), RESET; print " - ".lang('arg', 'contact', 'seszu (seba@kazuko.pl)').".\n";
			print "\n";
			print "  ".lang('other', 'changes_in')." ".$aCurrent[0]." [".$aCurrent[1]."]:\n";
			   for($i=2; $i<$zCurrent; $i++) {
				print "   ".trim($aCurrent[$i])."\n";
			   }
		   }

	   } else {
		   if(!$resCurrent->is_success and $resPrevious->is_success) {
			print $resCurrent->status_line, "\n";
		   } elsif($resCurrent->is_success and !$resPrevious->is_success) {
			print $resPrevious->status_line, "\n";
		   } else {
			print $resCurrent->status_line, "\n";
			print $resPrevious->status_line, "\n";
		   }
	   }

	print "\n";
	exit 0;
   }


   if(!$quiet) {
	print " Alert for LMS, version $p_version [$p_data]\n";
	print "   CTRL+C - anuluj\n";
	print "\n";
   }


   if(!$configfile) { $configfile = "/etc/lms/lms.ini"; }

   if(! -r $configfile and -f $configfile) {
	   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (nie można odczytać pliku $configfile)\n"; print BOLD RED, "\n\n\tSTOP PROGRAM!\n\n", RESET; }
	exit 1;
   } elsif(! -f $configfile) {
	   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (plik $configfile nie istnieje)\n"; print BOLD RED, "\n\n\tSTOP PROGRAM!\n\n", RESET; }
	exit 1;
   }
my $ini = new Config::IniFiles -file => $configfile;
print @Config::IniFiles::errors;


my $dbType   = getVal('database', 'type', $ini) || 'mysql';
my $dbHost   = getVal('database', 'host', $ini) || 'localhost';
my $dbUser   = getVal('database', 'user', $ini) || 'root';
my $dbPasswd = getVal('database', 'password', $ini) || '';
my $dbName   = getVal('database', 'database', $ini) || 'lms';


my $lastOnline  = getVal('alert', 'lastonline', $ini) || '300';			# po jakim czasie uznawane że stacja niedziałą
my $fpingBinary = getVal('alert', 'fping-binary', $ini) || "/usr/bin/fping";
my $alertPoRaz2  = getVal('alert', 'alertporaz2', $ini) || 'false';

my $alertMail = $argAlertMail ? "true" : "false";
my $listMail  = getVal('alert', 'list-mail', $ini) || '';

my $smtpHost   = getVal('alert', 'smtp-host', $ini) || '';
my $smtpLogin  = getVal('alert', 'smtp-login', $ini) || '';
my $smtpPasswd = getVal('alert', 'smtp-passwd', $ini) || '';
my $smtpAuth   = getVal('alert', 'smtp-auth', $ini) || 'PLAIN';	# LOGIN, PLAIN
my $smtpPort   = getVal('alert', 'smtp-port', $ini) || '587';		# 25, 587, 465

my $fromMail   = podmien(getVal('alert', 'from-mail', $ini), $ini) || $smtpLogin;
my $fromMailOrg = $fromMail;
my $fromName   = podmien(getVal('alert', 'from-name', $ini), $ini) || $smtpLogin;
my $mailTemat  = getVal('alert', 'mail-temat', $ini) || "alert-lms";


my $alertTel  = $argAlertTel ? "true" : "false";
my $listTel   = getVal('alert', 'list-tel', $ini) || '';

my $modemPort  = getVal('alert', 'modem-port', $ini) || "/dev/ttyUSB0";		# ls -l /dev/ttyUSB* && ls -l /dev/ttyACM*
my $modemPIN   = getVal('alert', 'modem-pin', $ini) || "";



my $dbase;
my $utsfmt;
my $x2;
my $update;

   if($dbType =~ /mysql/){
	$dbase = DBI->connect("DBI:mysql:database=$dbName;host=$dbHost","$dbUser","$dbPasswd", { RaiseError => 1 });	# PrintError => 0, AutoCommit => 0
	$utsfmt = "UNIX_TIMESTAMP()";
	$dbase->do('SET NAMES utf8');

   } elsif($dbType eq "postgres") {
	$dbase = DBI->connect("DBI:Pg:dbname=$dbName;host=$dbHost","$dbUser","$dbPasswd", { RaiseError => 1 });
	$utsfmt = "EXTRACT(EPOCH FROM CURRENT_TIMESTAMP(0))";

   } else {
	print STDERR "Fatal error: unsupported database type: $dbType, exiting.\n";
	exit 1;
   }


  ### Dodawanie tabeli w której pamięta czy wysyłał alert
   if(!daneARR($dbase, "SHOW TABLES LIKE 'onion_netdevices'")) {
		my $idbq = $dbase->prepare("CREATE TABLE `onion_netdevices` ( 
							`id` int(11) NOT NULL AUTO_INCREMENT,
							`alert` varchar(13) NOT NULL DEFAULT '0',
							PRIMARY KEY (`id`)
						) ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_bin AUTO_INCREMENT=1;"); $idbq->execute();
   } else {
	   if(!danaDB($dbase, "SHOW COLUMNS FROM onion_netdevices WHERE Field='alert'", "Field")) {
		my $idbq = $dbase->prepare("ALTER TABLE `onion_netdevices` ADD `alert` VARCHAR( 13 ) NOT NULL DEFAULT '0'"); $idbq->execute();
	   }
   }




   if($argPing) {
	my @nodes = daneARR($dbase, "SELECT id, INET_NTOA(ipaddr) AS ip, name FROM vnodes WHERE ownerid=0 and access=1");
	   foreach my $node (@nodes) {
		   if(!$argGuiet) { print $node->{'ip'}."\t"; }
		my $Fping = qx($fpingBinary -ar3 $node->{'ip'} 2> /dev/null);
		   if($Fping) {
			my $idbq = $dbase->prepare("UPDATE nodes SET lastonline='".time()."' WHERE id=".$node->{'id'}); $idbq->execute();
			   if(!$argGuiet) { print GREEN, "ping ok!", RESET; print "\n"; }
		   } else {
			   if(!$argGuiet) { print RED, "host nieosiągalny!", RESET; print "\n"; }
		   }
	   }
   }



   if($lastOnline !~ /^\d+$/) {
	   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr lastonline musi być liczbą - sekcja alert w pliku $configfile)\n"; print BOLD RED, "\n\n\tSTOP PROGRAM!\n\n", RESET; }
	exit;
   }


   if(!$fpingBinary) {
	   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr fping-binary nie może być pusty - sekcja alert w pliku $configfile)\n";  print BOLD RED, "\n\n\tSTOP PROGRAM!\n\n", RESET; }
	exit;
   } else {
	   if(!(-f $fpingBinary)) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (plik $fpingBinary nie istnieje - parametr fping-binary, sekcja alert w pliku $configfile)\n";  print BOLD RED, "\n\n\tSTOP PROGRAM!\n\n", RESET; }
		exit;
	   } elsif(!(-r $fpingBinary)) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (nie można odczytać pliku $fpingBinary - parametr fping-binary, sekcja alert w pliku $configfile)\n";  print BOLD RED, "\n\n\tSTOP PROGRAM!\n\n", RESET; }
		exit;
	   }
   }




  ### Sprawdzanie parametrów maila
   if($alertMail eq "true") {
	   if(!$listMail) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr list-mail nie może być pusty - sekcja alert w pliku $configfile)\n"; }
		$alertMail = "false";
	   }

	   if(!$smtpHost) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr smtp-host nie może być pusty - sekcja alert w pliku $configfile)\n"; }
		$alertMail = "false";
	   }

	   if(!$smtpLogin) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr smtp-login nie może być pusty - sekcja alert w pliku $configfile)\n"; }
		$alertMail = "false";
	   }

	   if(!$smtpPasswd) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr smtp-passwd nie może być pusty - sekcja alert w pliku $configfile)\n"; }
		$alertMail = "false";
	   }

	   if($smtpAuth !~ /^(LOGIN|PLAIN|CRAM\-MD5|NTLM)$/) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr smtp-auth musi zawierać: LOGIN, PLAIN, CRAM-MD5 lub NTLM - sekcja alert w pliku $configfile)\n"; }
		$alertMail = "false";
	   }

	   if(!$smtpPort) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr smtp-port nie może być pusty - sekcja alert w pliku $configfile)\n"; }
		$alertMail = "false";
	   }

	   if($smtpPort !~ /^\d+$/) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr smtp-port musi być liczbą - sekcja alert w pliku $configfile)\n"; }
		$alertMail = "false";
	   }


	   if($fromMail !~ /^.+\@.+\..+$/) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr from-mail nie jest mailem - sekcja alert w pliku $configfile)\n"; }
		$alertMail = "false";
	   }
   }




  ### Sprawdzanie połaczenia z modemem
   if($alertTel eq "true") {
	   if(!$listTel) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr list-tel nie może być pusty - sekcja alert w pliku $configfile)\n"; }
		$alertTel = "false";
	   }

	   if(!$modemPort) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr modem-port nie może być pusty - sekcja alert w pliku $configfile)\n"; }
		$alertTel = "false";
	   }

	   if($modemPIN and $modemPIN !~ /^\d+$/) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (parametr modem-pin musi być liczbą - sekcja alert w pliku $configfile)\n"; }
		$alertTel = "false";
	   }

	plikZapisz("/tmp/gammu_conf", "[gammu]\nport = $modemPort\nconnection = at");

	my $out = qx(gammu -c /tmp/gammu_conf identify);

	my %data = ();
	   if($out =~ /IMEI/) {
		   if(!($out =~ /SIM\sIMSI/)) {
			system qx( gammu -c /tmp/gammu_conf --entersecuritycode PIN $modemPIN 1>> /dev/null 2>> /dev/null);
		   }
		$out = qx(gammu -c /tmp/gammu_conf identify);
		   if(!($out =~ /SIM\sIMSI/)) {
			   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (niepoprawny PIN - sekcja alert, parametr modem-pin w pliku $configfile)\n"; }
			$alertTel = "false";
		   }

	   } elsif(($out =~ /Brak/ or $out =~ /Error/)) {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (prawdopodobnie źle podany ściezka w parametrze modem-port sekcja alert w pliku $configfile)\n"; }	# print "Odpowiedź od gammu:\n$out\n";
		$alertTel = "false";
	   } else {
		   if(!$argGuiet) { print RED, " ERROR!", RESET; print "\n (brak programu gammu)\n"; }
		$alertTel = "false";
	   }
   }





   if($alertMail eq "true" or $alertTel eq "true") {

	   if($alertMail eq "true") {
		   if($fromMail) { $fromMail = encode_header($fromName)." <$fromMail>"; }
	   }

	my %on = ();
	my $time = time();
	my %mARR = (0 => "działa", 1 => "nie działa", 2 => "nie działa");
	my %sARR = (0 => 1, 1 => 0, 2 => 0);

	my @netdevices = daneARR($dbase, "SELECT n.id, n.name, o.id AS oid, o.alert FROM netdevices n LEFT JOIN onion_netdevices o ON (o.id = n.id)", "++", "id");
	my @netlinks = daneARR($dbase, "SELECT * FROM netlinks", "++", "src");

	my %status = ();

	   foreach my $netdev (@netdevices) { if($netdev->{'id'}) {
		   if(!$netdev->{'oid'}) {
			my $idbq = $dbase->prepare("INSERT INTO onion_netdevices SET id='".$netdev->{'id'}."', alert='0'"); $idbq->execute();
		   }

		my @out = daneARR($dbase, "SELECT COUNT(*) AS wsio, SUM(CASE WHEN lastonline > $time - $lastOnline THEN 1 ELSE 0 END) AS online FROM vnodes WHERE ownerid=0 and access=1 and netdev=".$netdev->{'id'});

		   if($out[0]{'wsio'}) {
			   if($out[0]{'wsio'} eq $out[0]{'online'}) {
				$status{$netdev->{'id'}} = 1;
			   } elsif($out[0]{'wsio'} ne $out[0]{'online'}) {
				$status{$netdev->{'id'}} = 0;
			   }
		   }
	   }}



	my %alert = ();
	   foreach $i (keys %status) {
		my @alert = split(/\|/, $netdevices[$i]{'alert'});

		   if($status{$i} and $alert[0]) {
			$alert{$i} = 0;
		   } elsif(!$status{$i} and $alert[0] eq 0) {
			$alert{$i} = 1;
		   } elsif(!$status{$i} and $alert[0] eq 1 and $alertPoRaz2+$alert[1] < $time and $alertPoRaz2 ne "false") {
			$alert{$i} = 2;
			   if($x2 !~ /^.+x2$/) { $x2 = " po x2"; }
		   }
	   }


	my %sendSMS = ();
	my %sendMAIL = ();

	   foreach $i (keys %alert) { 
		my $iold;
		   for($in = 1; $in < 10; $in) {
			if($iold) { $i = $iold; }
			my $dst = @netlinks->[$i]{'dst'};

			   if($dst and defined($alert{$dst})) {
				$iold = $dst;
			   } elsif(defined($alert{$i})) {
				my @nodes = daneARR($dbase, "SELECT id, name, INET_NTOA(ipaddr) AS ip, (CASE WHEN lastonline > $time - $lastOnline THEN 1 ELSE 0 END) AS online FROM vnodes WHERE ownerid=0 and access=1 and netdev='$i' and (CASE WHEN lastonline > $time - $lastOnline THEN 1 ELSE 0 END)=0", "++", "id");
				my $cache = "";
				   foreach my $node (@nodes) { if($node->{'id'}) { $cache .= "  - ".$node->{'name'}." (".$node->{'ip'}.")\n"; }}

				$sendSMS{$i} = @netdevices->[$i]{'name'}." (".$sARR{$alert{$i}}.")";
				$sendMAIL{$i} = @netdevices->[$i]{'name'}." (".$mARR{$alert{$i}}.")".($cache ? "\n".$cache."\n" : "");
				last;
			   }
		   }
	   }


	my $send = ();
	   foreach $is (keys %sendSMS) { $send->{'sms'} .= $sendSMS{$is}.";\n"; }
	   foreach $im (keys %sendMAIL) { $send->{'mail'} .= $sendMAIL{$im}."\n"; }
	$send->{'sms'} =~ s/\s$//g;



	   if($alertMail eq "true" and $send->{'sms'} and $send->{'mail'} and $fromMail) {
		my @maill = split(/\@/, $fromMailOrg);

		   foreach my $toMail (split(/\s/, $listMail)) {
			$toMail = trim($toMail);
			   if(!$argGuiet) { print "MAIL --> ".$toMail."\t"; }
			   if($toMail =~ /^[a-zA-Z0-9\.\-_\+]+\@[a-zA-Z0-9\.\-_]+\.[a-z]{2,4}$/) {
				my $sender = new Mail::Sender;
				   if($toMail =~ /^\+*\d{9,11}.*$/) {
					   if(ref($sender->MailMsg({
						smtp => $smtpHost,
						from => $fromMail,
						to => $toMail,
						subject => strftime('%d-%m-%Y %H:%M:%S',localtime($time)).$x2,
						msg => delPL($send->{'sms'}),
						auth => $smtpAuth,
						authid => $smtpLogin,
						authpwd => $smtpPasswd,
						TLS_allowed => 0,
						messageid => "<".Email::MessageID->new( host => $maill[1] ).">",
					   }))) {
						$sender->Close;

						   if(!$update) {
							   foreach $i (keys %alert) {
								my $idbq = $dbase->prepare("UPDATE onion_netdevices SET alert='".($alert{$i} ? $alert{$i}."|".$time : "0")."' WHERE id=".$i); $idbq->execute();
							   }
							$update = 1;
						   }
						   if(!$argGuiet) { print GREEN, "ok\n", RESET; }
					   } elsif(!$argGuiet) {
						   if(!$argGuiet) { print RED, " ERROR!", RESET; print " ($Mail::Sender::Error)\n"; }
					   }

				   } else {
					   if(ref($sender->OpenMultipart({
						smtp => $smtpHost,
						from => $fromMail,
						to => $toMail,
						subject => $mailTemat." - ".strftime('%d-%m-%Y %H:%M:%S',localtime($time)).$x2,
						auth => $smtpAuth,
						authid => $smtpLogin,
						authpwd => $smtpPasswd,
						TLS_allowed => 0,
						messageid => "<".Email::MessageID->new( host => $maill[1] ).">",
					   }))) {

						$sender->Body({
							ctype => 'text/html', 
							encoding => 'quoted-printable', 
							charset => 'UTF-8', 
							msg => "<pre>".codePL($send->{'mail'})."</pre>\n", 
						});
						$sender->Close;

						   if(!$update) {
							   foreach $i (keys %alert) {
								my $idbq = $dbase->prepare("UPDATE onion_netdevices SET alert='".($alert{$i} ? $alert{$i}."|".$time : "0")."' WHERE id=".$i); $idbq->execute();
							   }
							$update = 1;
						   }
						   if(!$argGuiet) { print GREEN, "ok\n", RESET; }
					   } elsif(!$argGuiet) {
						   if(!$argGuiet) { print RED, " ERROR!", RESET; print " ($Mail::Sender::Error)\n"; }
					   }
				  }
			   } else {
				   if(!$argGuiet) { print RED, " ERROR!", RESET; print " (\"$toMail\" to nie jest mail)\n"; }
			   }

		   }
	   }


	   if($alertTel eq "true" and $send->{'sms'}) {
		   foreach my $toTel (split(/\s/, $listTel)) {
			   if($toTel =~ /^\+*\d{9,11}$/) {
				my $string = "(".strftime('%d-%m-%Y %H:%M:%S',localtime($time)).")$x2\n".$send->{'sms'};
				my $out = qx(gammu -c /tmp/gammu_conf sendsms TEXT $toTel -text "$string" 2>> /dev/null);
				   if(!$argGuiet) { print "SMS --> ".$toTel."\n"; }
				   if(!$update) {
					   foreach $i (keys %alert) {
						my $idbq = $dbase->prepare("UPDATE onion_netdevices SET alert='".($alert{$i} ? $alert{$i}."|".$time : "0")."' WHERE id=".$i); $idbq->execute();
					   }
					$update = 1;
				   }
			   }
		   }
	   }

   }






### FUNKCJE ###

sub podmien { 
 my($string, $ini) = @_; 
 my $smtpLogin = getVal('alert', 'smtp-login', $ini);

 $string =~ s/\%smtp\-login\%/$smtpLogin/g;

 return $string; 
}


sub trim($) {
 my $string = shift;
 $string =~ s/^\s+//;
 $string =~ s/\s+$//;
 return $string;
}


sub delHash { 
 my($tekst) = @_; 
 $tekst =~ s/\s+(#|;)(.*)$//g;
 $tekst =~ s/^"//g;
 $tekst =~ s/"$//g;
 return trim($tekst);
}


sub getVal { 
 my($key, $key2, $iniT) = @_; 

 return delHash($iniT->val($key, $key2));
}


# daneARR("SELECT * FROM statystyki WHERE id='1' LIMIT 1"); wynik array( 0 => array('xxx' => 'xxx', 'yy' => 'yy'));
sub daneARR {
 my($dbase, $prepare, $plus, $po) = @_;
 my @ARR;
 my $i = 1;

 my $sth = $dbase->prepare("$prepare")or die("Blad w zapytaniu(sub daneARR)!");
 $sth->execute();
   while( my $row = $sth->fetchrow_hashref()) { 
	   if($plus eq "+") { @ARR->[$i] = $row; $i++; } elsif($plus eq "++") { @ARR->[($row->{$po})] = $row; $i++; } else { push(@ARR, $row); }
   }
 $sth->finish();

 return @ARR;
}


# danaDB("SELECT * FROM interfaces WHERE id='1' LIMIT 1", "interfaces");
sub danaDB {
 my($dbase, $prepare, $danaX) = @_;

 my $sth = $dbase->prepare("$prepare")or die("Blad w zapytaniu(sub danaDB)!");
 $sth->execute();
 my $wynik = $sth->fetchrow_hashref();
 $sth->finish();

 return $wynik->{''.$danaX.''};
}


sub plikZapisz { 
 my($plik,$co) = @_; 
  open(UCHWYT, '>', $plik) or die "Nie można otworzyć $plik: $!"; 
  print UCHWYT $co; 
  close UCHWYT; 
}


sub codePL {
 my($txt) = @_;
 my %code = ("ą" => "&#261;", "ć" => "&#263;", "ę" => "&#281;", "ł" => "&#322;", "ń" => "&#324;", "ó" => "&#243;", "ś" => "&#347;", "ż" => "&#378;", "ź" => "&#380;",
	"Ą" => "&#260;", "Ć" => "&#262;", "Ę" => "&#280;", "Ł" => "&#321;", "Ń" => "&#323;", "Ó" => "&#242;", "Ś" => "&#346;", "Ż" => "&#377;", "Ź" => "&#379;",);
   foreach my $k (keys %code) {
	$txt =~ s/$k/$code{$k}/g;

   }
 return $txt;
}




sub delPL {
 my($tekst) = @_; 
 my %del = ('ą' => 'a', 'ć' => 'c', 'ę' => 'e', 'ł' => 'l', 'ń' => 'n', 'ó' => 'o', 'ś' => 's', 'ż' => 'z', 'ź' => 'z', 'Ą' => 'A', 'Ć' => 'C', 'Ę' => 'E', 'Ł' => 'L', 'Ń' => 'N', 'Ó' => 'O', 'Ś' => 'S', 'Ź' => 'Z', 'Ź' => 'Z');

   foreach my $k (keys %del) { $tekst =~ s/$k/$del{$k}/g; }

 return $tekst;
}


sub encode_header {
 my $subject = shift;
 $subject = encode_qp($subject, '');
 $subject =~ s/ /_/g;
 # @TODO: split long lines
 return "=?UTF-8?Q?$subject?=";
}


$dbase->disconnect();


   if(!$argQuiet) { print "done - ".(time()-$STARTA)." sec\n"; }
